
# coding: utf-8

# In[1]:


import networkx as nx
import matplotlib.pyplot as plt
from matplotlib import cm
import csv
import numpy as np
import math

from networkx.drawing.nx_agraph import write_dot, graphviz_layout
from itertools import count
from geopy.distance import vincenty
from scipy.stats.stats import pearsonr

import json
from geopy.geocoders import Nominatim


# # Load the stations associated each zip code

# In[2]:


# parse the station coordinates file and store in a dict
areas = {}
with open('AreasToStations.csv', 'rb') as csvfile:
    areas2St = csv.reader(csvfile, delimiter=',', quotechar='|')
    for row in areas2St:
        if row[0] == 'V1':
            #skip the header
            continue
        if row[3].replace('&','and') in areas:
            areas[row[3].replace('&','and')]['code'].append(row[2])
        else:
            areas[row[3].replace('&','and')] = {'lat':row[0], 'lon':row[1], 'code':[row[2]], 'name':row[3].replace('&','and')}


# # Load the passengers

# In[3]:


# parse the journeys file and store in a dict
passengers = []
with open('areas train dataset.csv', 'rb') as csvfile:
    areasTrain = csv.reader(csvfile, delimiter=',', quotechar='|')
    for row in areasTrain:
        passengers.append([row[0],row[1], int(row[2])])


# # Compute the journeys

# In[4]:


FIRST_STOP_ID = 9
LAST_STOP_ID = 107
COMPANY_NAME = 5

companies = set()
journeys = []
companies_journeys = {}

with open('AroundLondonJourneyMatrix.csv', 'rb') as csvfile:
    journeys_file = csv.reader(csvfile, delimiter=',', quotechar='|')
    for row in journeys_file:
        if row[1] == '"SourceStationCode"':
            # skip the header   
            continue
        if row[FIRST_STOP_ID] == '""':
            # skip empty journey  
            continue
        
        journey = [[row[FIRST_STOP_ID].replace('"',""), row[FIRST_STOP_ID + 1].replace('"',"").replace('&','and')]]
        companies.add(row[COMPANY_NAME])

        for stop in range(FIRST_STOP_ID + 2, LAST_STOP_ID + 1, 2):
            if row[stop] == '""' or row[stop] == 'NA':
                #journey completed
                break
            journey.append([row[stop].replace('"',""), row[stop + 1].replace('"',"").replace('&','and')])

        journeys.append(journey)
        if row[COMPANY_NAME] in companies_journeys:
            companies_journeys[row[COMPANY_NAME]].append(journey)
        else:
            companies_journeys[row[COMPANY_NAME]] = [journey]


# In[ ]:


def search_commuters(dataset, source, dest):
    for commuters in dataset:
        if commuters[0] == source and commuters[1] == dest:
            return commuters[2]
    return 0


# # Create the directed graph of all the journeys

# In[7]:


def create_DG_graph(journeys):
    DG = nx.DiGraph()
    idx = 1
    l = len(journeys)
    for journey in journeys:
        if not idx % 1000:
            print idx, " of ", l
        idx += 1
        if journey[0][1] in areas and journey[len(journey) - 1][1] in areas:
            source_area_codes = areas[journey[0][1]]['code']
            dest_area_codes = areas[journey[len(journey) - 1][1]]['code']
            number_of_commuters = 0
            for s_code in source_area_codes:
                for d_code in dest_area_codes:
                    number_of_commuters += search_commuters(passengers, s_code, d_code)    
            for i in range(len(journey) - 1):
                num = number_of_commuters
                if num > 0:
                    if DG.has_edge(journey[i][1], journey[i + 1][1]):
                        num += DG.get_edge_data(journey[i][1], journey[i + 1][1])['p']
                    DG.add_edge(journey[i][1], journey[i + 1][1], p = num)
    return DG

DG = create_DG_graph(journeys)


# In[8]:


def create_FULL_DG_graph(journeys): 
    FULL_DG = nx.DiGraph()
    idx = 1
    l = len(journeys)
    for journey in journeys:
        if not idx % 1000:
            print idx, " of ", l
        idx += 1
        if journey[0][1] in areas and journey[len(journey) - 1][1] in areas:
            source_area_codes = areas[journey[0][1]]['code']
            dest_area_codes = areas[journey[len(journey) - 1][1]]['code']
            number_of_commuters = 0
            for s_code in source_area_codes:
                for d_code in dest_area_codes:
                    number_of_commuters += search_commuters(passengers, s_code, d_code)    
            for i in range(len(journey) - 1):
                num = number_of_commuters
                if FULL_DG.has_edge(journey[i][1], journey[i + 1][1]):
                    num += FULL_DG.get_edge_data(journey[i][1], journey[i + 1][1])['p']
                FULL_DG.add_edge(journey[i][1], journey[i + 1][1], p = num)

    return FULL_DG

#FULL_DG = create_FULL_DG_graph(journeys)


# # Compute nodes coordinates

# In[11]:


# load the gps coorinates
gps_coordinates = {}
with open('StationsCoordinates.csv', 'rb') as csvfile:
    areas2St = csv.reader(csvfile, delimiter=',', quotechar='|')
    for row in areas2St:
        if row[0] == '"lat"':
            #skip the header
            continue
        gps_coordinates[row[2].replace('"','').encode('ascii',errors='ignore')] = [row[2].replace('"','').encode('ascii',errors='ignore'),  float(row[0].replace('"','')),  float(row[1].replace('"',''))]

# load other gps coorinates
with open('OtherStations.csv', 'rb') as csvfile:
    areas2St = csv.reader(csvfile, delimiter=',', quotechar='|')
    for row in areas2St:
        if row[0] == '"lat"':
            #skip the header
            continue
        gps_coordinates[row[0].replace('"','').encode('ascii',errors='ignore')] = [row[0].replace('"','').encode('ascii',errors='ignore'), float(row[1].replace('"','')), float(row[2].replace('"',''))]


# # Draw the journeys graph

# In[12]:


egreen=[(u,v) for (u,v,d) in DG.edges(data=True) if 'p' in d and d['p']<=10]
eorange=[(u,v) for (u,v,d) in DG.edges(data=True) if 'p' in d and d['p']>10 and d['p']<=50]
ered=[(u,v) for (u,v,d) in DG.edges(data=True) if 'p' in d and d['p']>50 and d['p']<=100]
eblack=[(u,v) for (u,v,d) in DG.edges(data=True) if 'p' in d and d['p']>100]


# In[13]:


#pos=nx.spring_layout(DG, k=1) # positions for all nodes
pos=nx.circular_layout(DG) # positions for all nodes

# nodes
nx.draw_networkx_nodes(DG,pos,node_size=2, node_color='blue')

# edges
nx.draw_networkx_edges(DG,pos,edgelist=ered, width=0.1, edge_color='red')
nx.draw_networkx_edges(DG,pos,edgelist=egreen, width=0.1, edge_color='green')
nx.draw_networkx_edges(DG,pos,edgelist=eorange, width=0.1, edge_color='orange')
nx.draw_networkx_edges(DG,pos,edgelist=eblack, width=0.1)

# labels
nx.draw_networkx_labels(DG,pos,font_size=2,font_family='sans-serif')

plt.axis('off')

plt.savefig("Graph.png", format="PNG", dpi=1000)
plt.show()


# In[14]:


ratio = []
for node in DG.nodes():
    ratio.append([node, DG.out_degree(node) + 1 / float(DG.in_degree(node) + 1)])



# In[15]:


ratio.sort(key=lambda tup: tup[1])
plt.plot([r[1] for r in ratio])
plt.show()


# In[16]:


DIFF_THRESHOLD = 0.3

ratio.sort(key=lambda tup: tup[1])

ratio_diff = []
for i in range(len(ratio) - 1):
    ratio_diff.append(ratio[i + 1][1] - ratio[i][1])
    
plt.figure(figsize=(15,5))
plt.plot(ratio_diff)
plt.axhline(y=DIFF_THRESHOLD, color='red')
plt.show()


# # Compute the weighted ratio - out passengers / in passengers

# In[17]:


w_ratio = []
for node in DG.nodes():
    out_weighted_degree = 1
    in_weighted_degree = 1
    
    for e in DG.out_edges(node, data=True):
        out_weighted_degree += e[2]['p']
    
    for e in DG.in_edges(node, data=True):
        in_weighted_degree += e[2]['p']
    
    w_ratio.append([node, out_weighted_degree / float(in_weighted_degree)])
    


# In[18]:


w_ratio.sort(key=lambda tup: tup[1])
plt.plot([r[1] for r in w_ratio])
plt.show()


# In[19]:


aw_ratio = []
for node in DG.nodes():
    out_weighted_degree = 1
    in_weighted_degree = 1
    
    for e in DG.out_edges(node, data=True):
        out_weighted_degree += e[2]['p']
    
    for e in DG.in_edges(node, data=True):
        in_weighted_degree += e[2]['p']
    
    
    out_weighted_degree /= float(DG.out_degree(node) + 1)
    in_weighted_degree /= float(DG.in_degree(node) + 1)

    aw_ratio.append([node, out_weighted_degree  / float(in_weighted_degree)])
    


# In[20]:


aw_ratio.sort(key=lambda tup: tup[1])
plt.plot([r[1] for r in aw_ratio])
plt.show()


# In[21]:


def get_ratio(array, name):
    for node in array:
        if node[0] == name:
            return node[1]
    return None


# In[23]:


ratio.sort(key=lambda tup: tup[0])
w_ratio.sort(key=lambda tup: tup[0])
aw_ratio.sort(key=lambda tup: tup[0])

comp = zip([r[1] for r in ratio], [r[1] for r in w_ratio], [r[1] for r in aw_ratio])

comp.sort(key=lambda tup: tup[1])

plt.figure(figsize=(15,5))
plt.plot(comp)
plt.show()


# In[24]:


ratio.sort(key=lambda tup: tup[0])
w_ratio.sort(key=lambda tup: tup[0])
aw_ratio.sort(key=lambda tup: tup[0])

plt.xscale('log')
plt.yscale('log')

plt.scatter([r[1] for r in aw_ratio], [r[1] for r in w_ratio])


plt.show()


# In[26]:


def get_basal_and_top_nodes(GG):
    basal_nodes = []
    for node in GG.nodes():
        if GG.in_degree(node) == 0:
            basal_nodes.append(node)

    top_nodes = []
    for node in GG.nodes():
        if GG.out_degree(node) == 0:
            top_nodes.append(node)
    return basal_nodes, top_nodes


# In[27]:


basal_nodes, top_nodes = get_basal_and_top_nodes(DG)
print 'basal nodes:'
print basal_nodes
print 'top nodes:'
print top_nodes


# # Compute the NET network

# In[28]:


NET_THRESHOLD = 10
def compute_net_network(DG, threshold):
    DG_net=DG.copy()
    for n1 in DG_net.nodes():
        for n2 in DG_net.nodes():

            if n1 in DG_net.neighbors(n2) and n2 in DG_net.neighbors(n1):
                p1 = DG.get_edge_data(n1,n2)['p']
                p2 = DG.get_edge_data(n2,n1)['p']
                if p2 > 0 and p1/float(p2) <= threshold:
                     DG_net.remove_edge(n1,n2)
                if p1 > 0 and p2/float(p1) <= threshold:
                     DG_net.remove_edge(n2,n1)
    return DG_net


# In[29]:


DG_net = compute_net_network(DG, NET_THRESHOLD)                
basal_nodes, top_nodes = get_basal_and_top_nodes(DG_net)
print "basal nodes: ", len(basal_nodes)
print "top nodes: ", len(top_nodes)


# # Remove disconnected nodes

# In[30]:


def remove_disconnected_nodes(DG_net):
    DG2=DG_net.copy()
    nodes_to_remove = True
    while(nodes_to_remove):
        basal_nodes, top_nodes = get_basal_and_top_nodes(DG2)
        nodes_to_remove = False
        removal_list = []
        for node in DG2.nodes():
            if node in basal_nodes:
                continue
            connected_node = False
            for bn in basal_nodes:
                connected_node = nx.has_path(DG2,bn,node) or connected_node
            if not connected_node:
                removal_list.append(node)
                nodes_to_remove = True

        for node in removal_list:
            DG2.remove_node(node)
    return DG2



# In[31]:


DG2 = remove_disconnected_nodes(DG_net)


# # Compute trophic levels

# In[33]:


# returns trophic levels and annotated Directed Graph
def compute_trophic_levels(DG):
    adj_mtrx = nx.to_numpy_matrix(DG)
    Kv = np.array([max(DG.in_degree(node), 1) for node in DG.nodes()])
    A = np.diag(Kv) - np.transpose(adj_mtrx)
    S = None
    if np.linalg.det(A) == 0:
        print 'Error: Singular matrix'
    else:
        S = np.linalg.solve(A, Kv)
        trophic_levels = []
        idx = 0
        for node in DG.nodes():
            trophic_levels.append([node, S[idx]])
            DG.nodes()[node]['tr_lvl'] = float(S[idx])
            idx += 1
    
    return S, DG


# In[34]:


S, DG2 = compute_trophic_levels(DG2)


# # Set fixed number of trophic levels

# In[35]:


def set_basal_nodes(DG, number_basals = 10):
    DG_filt = DG.copy()
    DG_filt = sorted(nx.strongly_connected_component_subgraphs(DG_filt),key=len,reverse=True)[0]
    in_s = dict(DG_filt.in_degree(weight='p'))
    in_s_sort = sorted(in_s,key=in_s.get)
    for i in range(number_basals):
        for e in DG_filt.in_edges(in_s_sort[i]):
            DG_filt.remove_edge(e[0],e[1])
    return DG_filt


# # Create DG graph for each company

# In[38]:


DG_companies = {}
for c in companies:
    DG_companies[c] = create_DG_graph(companies_journeys[c])


# # Geographic visualization

# In[39]:


def normalize_lvl(lvl, max_lvl):
    a = lvl / float(max_lvl) * 100
    return format(a, '.2f')
    
def draw_geo_graph(DG, gps_coordinates, trophic_levels = True):
    pos = {}
    for node in DG.nodes(data=True):
        if node[0] in gps_coordinates:
            a = (gps_coordinates[node[0]][2], gps_coordinates[node[0]][1])    
            pos[node[0]] = a
        else:
            print node[0]
    
    plt.figure(figsize=(25,25))
    plt.axis('off')

    if trophic_levels:

        max_lvl = max([node[1]['tr_lvl'] for node in DG.nodes(data=True)])
        groups = set(normalize_lvl(lvl, max_lvl) for lvl in nx.get_node_attributes(DG,'tr_lvl').values())
        mapping = dict(zip(sorted(groups), count()))
        colors = [mapping[normalize_lvl(DG.node[n]['tr_lvl'], max_lvl)] for n in DG.nodes()]
        
        nx.draw_networkx_nodes(DG, pos, with_labels=None, node_color=colors, cmap=cm.coolwarm)

        nx.draw_networkx_edges(DG, pos)
        nx.draw_networkx_labels(DG, pos)

        #plt.colorbar()
    else:
        nx.draw(DG, pos,node_size=100, with_labels=False)
        
    plt.show()
    return pos


# In[40]:


draw_geo_graph(DG, gps_coordinates, False)


# In[41]:


for c in companies:
    print(c)
    draw_geo_graph(DG_companies[c], gps_coordinates, False)
    


# In[42]:


# PPM, CaSL, new Casl, xxx

delays = {}
delays['"London Overground"'] = [94.5, 2, 2.3, 1.37]
delays['"Chiltern Railways"'] = [93.1, 1, 1.8, 1.14]
delays['"c2c"'] = [94.5, 3.3, 2.2, 1.42]
delays['"South West Trains"'] = [87.1, 2.8, 3.6, 1.98]
delays['"London Midland"'] = [88.8, 2.9, 2.9, 2.95]
delays['"Southeastern"'] = [86.1, 2.9, 4, 1.92]
delays['"London Underground"'] = [None, None, None, None]
delays['"TfL Rail"'] = [94.7, 2.7, 2.7, 1.15]
delays['"Great Northern"'] = [None, None, None, None]
#delays['"Heathrow Express"'] = [89.8, 0.7, None]
delays['"Southern"'] = [None, None, None, None]
delays['"Great Western Railway"'] = [86.6, 2.3, 3, 2.67]  #old 88.4
delays['"Greater Anglia"'] = [88, 2.9, 3.1, 2.49] # old 88.8
delays['"Heathrow Connect"'] = [None, None, None, None]
delays['"Thameslink"'] = [None, None, None, None]

for k,v in delays.iteritems():
    if delays[k][0]:
        delays[k][0] = 100 - delays[k][0]
        
        
# Number of Employees, Stations, Trains planned, Passengers Journeys (millions)
info = {}
info['"London Overground"'] = [1413, 81, 490356, 188.8]
info['"Chiltern Railways"'] = [843, 35, 142861, 26.4]
info['"c2c"'] = [652, 25, 127099, 46.6]
info['"South West Trains"'] = [5055, 186, 593623, 229.9]
info['"London Midland"'] = [2563, 150, 423884, 73.6]
info['"Southeastern"'] = [4362, 165, 651056, 182.4]
info['"London Underground"'] = [None, None, None, None]
info['"TfL Rail"'] = [754, 12, 82130, 44.6]
info['"Great Northern"'] = [None, None, None, None] 
#delays['"Heathrow Express"'] = [89.8, 0.7, None]
info['"Southern"'] = [None, None, None, None]
info['"Great Western Railway"'] = [6105, 207, 529623, 104.9]
info['"Greater Anglia"'] = [2903, 132, 440469, 82.0]
info['"Heathrow Connect"'] = [None, None, None, None]
info['"Thameslink"'] = [None, None, None, None]      


# In[43]:


del_array = []
for key, value in delays.iteritems():
    if value[0]:
        temp = [key,value]
        del_array.append(temp)
    
del_array = sorted(del_array, key=lambda delay: delay[1][0], reverse=True)


fig, ax1 = plt.subplots(figsize=(10, 5))

ax1.set_xticks([x for x in range(len(del_array))], minor=False)
ax1.set_xticklabels([d[0] for d in del_array], fontdict=None, minor=False, rotation='vertical')


ax1.plot([d[1][0] for d in del_array], 'b')
ax1.set_ylabel('ppm', color='b')
ax1.tick_params('y', colors='b')

ax2 = ax1.twinx()

ax2.plot([d[1][1] for d in del_array], 'r')
ax2.set_ylabel('CaSL', color='r')
ax2.tick_params('y', colors='r')

ax3 = ax1.twinx()

ax3.plot([d[1][2] for d in del_array], 'g')
ax3.set_ylabel('Avg lateness', color='g')
ax3.tick_params('y', colors='g')

fig.tight_layout()

plt.show()


# In[44]:


threshold = 1
for c in companies:
    print(c)
    DG_net = compute_net_network(DG_companies[c], threshold)
    DG2 = remove_disconnected_nodes(DG_net)
    S, DG2 = compute_trophic_levels(DG2)

    #DG_filt = set_basal_nodes(DG, number_basals = 50)
    #S, DG2 = compute_trophic_levels(DG_filt)

    pos = draw_geo_graph(DG2, gps_coordinates)


# # Passengers flows vs trophic level

# In[45]:


def weights(DG):
    nodes = DG.nodes(data=True)
    in_dgr = DG.in_degree(weight='p')
    out_dgr = DG.out_degree(weight='p')

    vals = []
    for node in DG.nodes(data=True):
        vals.append([node[0], node[1]['tr_lvl'], in_dgr[node[0]], out_dgr[node[0]]])
    return vals


DG_filt = set_basal_nodes(DG, number_basals = 300)
S, DG_filt = compute_trophic_levels(DG_filt)

weights_arr = weights(DG_filt)
plt.figure(figsize=(15,5))
plt.xlabel('trophic level')
plt.scatter([n[1] for n in weights_arr], [n[2] for n in weights_arr], s=10)
plt.scatter([n[1] for n in weights_arr], [n[3] for n in weights_arr], c = 'red', s=10)
plt.show()

plt.figure(figsize=(15,5))
plt.ylabel('Passengers out / passengers in')
plt.xlabel('Trophic level')
plt.scatter([n[1] for n in weights_arr], [n[3]/float(n[2]) if n[2] != 0 else 0 for n in weights_arr], c = 'green', s=10)
plt.show()

plt.figure(figsize=(15,5))
plt.xlabel('trophic level')
plt.scatter([n[1] for n in weights_arr], [n[3]/float(n[2]) if n[2] != 0 else 0 for n in weights_arr], c = 'green', s=10)
plt.ylim(0,2.5)
plt.show()


# # Trophic difference

# In[47]:


def trophic_diff(DG):
    nodes = DG.nodes(data=True)
    trophic_diff = []
    for edge in DG.edges():
        t_diff = abs(nodes[edge[0]]['tr_lvl'] - nodes[edge[1]]['tr_lvl'])
        trophic_diff.append([edge, t_diff])
    return trophic_diff

t_diff = trophic_diff(DG2)
print t_diff


# # Euclidean distance

# In[48]:


def euclidean_dist(DG):
    nodes = DG.nodes(data=True)
    e_diff = []
    for edge in DG.edges():
        a = (gps_coordinates[edge[0]][1], gps_coordinates[edge[0]][2])
        b = (gps_coordinates[edge[1]][1], gps_coordinates[edge[1]][2])
        g_diff = vincenty(a, b).miles
        e_diff.append([edge, g_diff])
    return e_diff

e_dist = euclidean_dist(DG2)


# # Trophic difference / euclidean distance

# In[49]:


plt.figure(figsize=(10,5))
plt.xlabel('Trophic difference')
plt.ylabel('Euclidian distance')
#plt.xscale('log')
#plt.xlim(0,30000)
plt.scatter([t[1] for t in t_diff], [t[1] for t in e_dist])
plt.show()



# # Incoherence parameter

# In[50]:


def incoherence_parameter(DG, trophic_diff):
    q = 0
    for x in trophic_diff:
        q += x[1] * x[1]
    q /= float(DG.number_of_edges())
    q = math.sqrt(q - 1)
    return q

q = incoherence_parameter(DG2, trophic_diff)
print q


# # Basal ensemble expectation

# In[51]:


def basal_ensemble_expectation(DG, q):
    # number of edges connected to basal nodes
    L_b = 0
    basal_nodes, _ = get_basal_and_top_nodes(DG)
    for node in basal_nodes:
        L_b += len(DG.out_edges(node))

    q_b = math.sqrt(DG.number_of_edges() / float(L_b) - 1) 
    return q_b


q_b = basal_ensemble_expectation(DG, q)
q_ratio = q / q_b

print "Incoherence parameter: ", q
print "Basal ensemble expectation: ", q_b
print "ratio q/qñ: ", q_ratio


# # Number of trophic levels

# In[52]:


unique, counts = np.unique(S, return_counts=True)
print "Number of levels: ", len(unique)

plt.figure(figsize=(15,5))
plt.plot(unique)
plt.show()


# In[53]:


w_ratio = []
for node in DG2.nodes():
    out_weighted_degree = 1
    in_weighted_degree = 1
    
    for e in DG2.out_edges(node, data=True):
        out_weighted_degree += e[2]['p']
    
    for e in DG2.in_edges(node, data=True):
        in_weighted_degree += e[2]['p']
    
    w_ratio.append([node, out_weighted_degree / float(in_weighted_degree)])
 


# # Flow threshold

# In[54]:


flow_q = []
flow_q_b = []
flow_q_ratio = []

for t in range(0,101):
    threshold = t * 0.1
    DG_net = compute_net_network(DG, threshold)
    DG2 = remove_disconnected_nodes(DG_net)
    S, DG2 = compute_trophic_levels(DG2)

    basal_nodes, top_nodes = get_basal_and_top_nodes(DG2)

    t_diff = trophic_diff(DG2)
    flow_q.append(incoherence_parameter(DG2, t_diff))
    flow_q_b.append(basal_ensemble_expectation(DG2, flow_q))



    print '*************', t, '******************'
 #   print 'Number of basal nodes: ', len(basal_nodes)
 #   print 'Number of top nodes: ', len(top_nodes)
 #   print 'Disconnected nodes: ', DG_net.number_of_nodes() -  DG2.number_of_nodes()

 #   print "Threshold: ", threshold
 #   print "Incoherence parameter: ", q[-1]
#     print "Basal ensemble expectation: ", q_b
#     print "ratio q/qñ: ", q_ratio


# # Incoherence parameter (q) graph

# In[55]:


plt.figure(figsize=(15,5))
plt.xlabel('Threshold filter')
plt.ylabel('incoherence parameter')
plt.plot([t * 0.1 for t in range(20, len(flow_q))], flow_q[20:])
plt.show()


# # ratio q/qñ

# In[56]:


plt.figure(figsize=(15,5))
plt.xlabel('Threshold filter')
plt.ylabel('q/q^ ratio')
plt.plot([t * 0.1 for t in range(20,len(flow_q))], [flow_q[i] / float(flow_q_b[i]) for i in range(20,len(flow_q))])
plt.show()


# # Basal nodes threshold

# In[57]:


enforc_q = []
enforc_q_b = []
enforc_q_ratio = []

for bn in range(1,51):

    DG_filt = set_basal_nodes(DG, number_basals = bn)
    #DG_filt = remove_disconnected_nodes(DG_filt)
    S, DG2 = compute_trophic_levels(DG_filt)

    basal_nodes, top_nodes = get_basal_and_top_nodes(DG2)

    t_diff = trophic_diff(DG2)
    enforc_q.append(incoherence_parameter(DG2, t_diff))
    enforc_q_b.append(basal_ensemble_expectation(DG2, enforc_q))


    print '**************',bn,'*****************'
    #print 'Number of basal nodes: ', len(basal_nodes)
    #print 'Number of top nodes: ', len(top_nodes)
    #print 'Size: ',  DG2.number_of_nodes()

    #print "Threshold: ", bn
    #print "Incoherence parameter: ", enforc_q[-1]
#     print "Basal ensemble expectation: ", enforc_q_b
#     print "ratio q/qñ: ", enforc_q_ratio


# In[58]:


qq = enforc_q[4:]
qq_b = enforc_q_b[4:]

plt.figure(figsize=(15,5))
plt.xlabel('Enforced basal nodes')
plt.ylabel('Incoherence parameter')
plt.plot(range(5, len(qq)+5), qq)
#plt.ylim(0,200)

plt.show()

plt.figure(figsize=(15,5))
plt.xlabel('Enforced basal nodes')
plt.ylabel('q/q^ ratio')

plt.plot(range(1, len(qq)+1), [qq[i] / float(qq_b[i]) for i in range(0, len(qq))])
plt.show()


# # TROPHIC COHERENCE - Merged

# In[118]:


# plt.figure(figsize=(15,5))
# plt.xlabel('Threshold filter')
# plt.ylabel('q/q^ ratio')

#plt.plot(range(1, len(enforc_q[4:])+1), [enforc_q[4:][i] / float(enforc_q_b[4:][i]) for i in range(0, len(enforc_q[4:]))])

#plt.plot([t * 0.05 for t in range(20,len(flow_q))], [flow_q[i] / float(flow_q_b[i]) for i in range(20,len(flow_q))])



fig, ax1 = plt.subplots(figsize=(10, 8))

#ax1.set_xticks(range(1, len(enforc_q[4:])+1), minor=False)
#ax1.set_xticklabels([d[0].replace('"','') for d in del_array_q], fontdict=None, minor=False, rotation='vertical')

ax1.plot([t * 0.1 for t in range(20,len(flow_q))], [flow_q[i] / float(flow_q_b[i]) for i in range(20,len(flow_q))], 'b')
ax1.set_xlabel('Passenger flow filter', color='b', fontsize=24)
ax1.tick_params('x', colors='b', labelsize = 24)

ax2 = ax1.twiny()
ax2.plot(range(1, len(enforc_q[4:])+1), [enforc_q[4:][i] / float(enforc_q_b[4:][i]) for i in range(0, len(enforc_q[4:]))], 'r')
ax2.set_xlabel('Enforced basal nodes', color='r', fontsize=24)
ax2.tick_params('x', colors='r', labelsize = 24)

ax1.tick_params('y', labelsize = 24)
ax1.set_ylabel('q/q^ ratio', fontsize=24)

fig.tight_layout()



# # Betweenness centrality

# In[60]:


def get_betweenness_core_periphery(DG):
    k_vector = []
    rank = nx.betweenness_centrality(DG)
    for node in DG.nodes():
        k_p = 0
        lvl = rank[node]
        for edge in DG.out_edges(node):
            if lvl <= rank[edge[1]]:
                k_p += 1
        k_vector.append([[node], k_p])

    k_vector.sort(key=lambda tup: rank[tup[0][0]], reverse=True)

    return k_vector


# In[61]:


def get_degree_core_periphery(DG):
    DG = DG.to_undirected()
    rank = []
    for node in DG.nodes():
        rich_friends = 0
        for edge in DG.edges(node):
            if DG.degree(edge[1]) >= DG.degree(node):
                rich_friends += 1
        rank.append([node, rich_friends, DG.degree(node)])
    rank = sorted(rank, key=lambda x: x[2], reverse = True)
    return rank


# # Rank trophic levels

# In[62]:


def get_trophic_core_periphery(DG):
    k_vector = []
    for node in DG.nodes(data=True):
        k_p = 0
        lvl = node[1]['tr_lvl']
        for edge in DG.out_edges(node[0]):
            if lvl <= DG.nodes(data=True)[edge[1]]['tr_lvl']:
                k_p += 1
        k_vector.append([node, k_p])
    k_vector.sort(key=lambda tup: tup[0][1]['tr_lvl'], reverse=True)

    return k_vector


# In[63]:


k_vector = get_trophic_core_periphery(DG2)
print  max(k_vector, key=lambda x: x[1])
print k_vector


# # Core-Periphery

# In[64]:


def core_periphery(DG, k_vector):
    core = []
    periphery = []
    is_core = True
    max_node = max(k_vector, key=lambda x: x[1])

    for node in k_vector:
        if is_core:
            core.append(node[0][0])
        else:
            periphery.append(node[0][0])            
        if node == max_node:
            is_core = False
    return core, periphery


def draw_core_graph(DG, k_vector, gps_coordinates):
    
    core = []
    periphery = []
    pos_core = {}
    pos_periphery = {}
    is_core = True
    max_node = max(k_vector, key=lambda x: x[1])
    for node in k_vector:
        a = (gps_coordinates[node[0][0]][2], gps_coordinates[node[0][0]][1])
        if is_core:
            pos_core[node[0][0]] = a
            core.append(node[0][0])
        else:
            pos_periphery[node[0][0]] = a
            periphery.append(node[0][0])            
        if node == max_node:
            is_core = False
    
    print "Core: ", len(core)
    print "Periphery: ", len(periphery)
    plt.figure(figsize=(60,60))
    plt.xlabel('Longitude')
    plt.ylabel('Latitude')

    nx.draw_networkx_nodes(DG, pos_core, nodelist=core, with_labels=True, node_color='black')
    nx.draw_networkx_nodes(DG, pos_periphery, nodelist=periphery, with_labels=True, node_color='red')

    plt.show()
    return pos


# # Core periphery graphs

# In[65]:


def count_core_periphery(k_vector):
    max_node = max(k_vector, key=lambda x: x[1])
    c = 0
    p = 0
    is_core = True
    for node in k_vector:
        if is_core:
            c += 1
        else:
            p += 1
            
        if node == max_node:
            is_core = False
    return (c, p)


# # Flows filter

# In[66]:


flow_betweenness_cp = []
flow_trophic_cp = []
flow_degree_cp = []
flow_rich_club = []

for t in range(0,101):
    threshold = t * 0.1
    
    DG_net = compute_net_network(DG, threshold)
    DG2 = remove_disconnected_nodes(DG_net)
    S, DG2 = compute_trophic_levels(DG2)

    basal_nodes, top_nodes = get_basal_and_top_nodes(DG2)
    k_bt = get_betweenness_core_periphery(DG2)
    k_tr = get_trophic_core_periphery(DG2)
    k_degr = get_degree_core_periphery(DG2)
    
    rc = None
    while rc is None:
        try:
            rc = nx.rich_club_coefficient(DG2.to_undirected(), normalized=True, Q=100)
        except ZeroDivisionError:
            print "Oops!  Error! I am trying again!"
    flow_betweenness_cp.append(count_core_periphery(k_bt))
    flow_trophic_cp.append(count_core_periphery(k_tr))
    flow_degree_cp.append(count_core_periphery(k_degr))
    flow_rich_club.append(rc)

    print "********",t,"********"
    #print "Threshold: ", threshold
    #print "core - periphery: ", trophic_cp[-1]


# In[68]:


rich_club_coeff = []
for x in flow_rich_club:
    k_range = range(0, max(x.keys())+1)
    rich_club_coeff.append(max([x[k] for k in k_range]))

print("Average", np.average(rich_club_coeff))
print("Min", np.min(rich_club_coeff))
print("Max", np.max(rich_club_coeff))


# In[69]:


plt.figure(figsize=(15,5))
plt.xlabel('Threshold filter')
plt.ylabel('Rich club')
for x in flow_rich_club:
    k_range = range(0, max(x.keys())+1)
    plt.plot(k_range, [x[k] for k in k_range])
plt.show()


plt.figure(figsize=(15,5))
plt.xlabel('Threshold filter')
plt.ylabel('Degree level core / periphery')
plt.plot([t * 0.1 for t in range(0, len(flow_degree_cp))], [t[0] / float(t[1]) if t[1] != 0 else 0 for t in flow_degree_cp])
plt.show()

plt.figure(figsize=(15,5))
plt.xlabel('Threshold filter')
plt.ylabel('Trophic level core / periphery')
plt.plot([t * 0.1 for t in range(0, len(flow_trophic_cp))], [t[0] / float(t[1]) if t[1] != 0 else 0 for t in flow_trophic_cp])
plt.show()

plt.figure(figsize=(15,5))
plt.xlabel('Threshold filter')
plt.ylabel('Betweenness core / periphery')
plt.plot([t * 0.1 for t in range(0, len(flow_betweenness_cp))], [t[0] / float(t[1]) if t[1] != 0 else 0 for t in flow_betweenness_cp], c='red')
plt.show()


plt.figure(figsize=(15,5))
plt.xlabel('Threshold filter')
plt.ylabel('Graph size')
plt.plot([t * 0.1 for t in range(0, len(flow_trophic_cp))], [t[0] + t[1] for t in flow_trophic_cp])
plt.show()


# # Basal nodes filter

# In[72]:


enforc_betweenness_cp = []
enforc_trophic_cp = []
enforc_degree_cp = []
enforc_rich_club = []


for bn in range(1, 51):
    DG_filt = set_basal_nodes(DG, number_basals = bn)
    S, DG2 = compute_trophic_levels(DG_filt)

    basal_nodes, top_nodes = get_basal_and_top_nodes(DG2)
    k_bt = get_betweenness_core_periphery(DG2)
    k_tr = get_trophic_core_periphery(DG2)
    k_degr = get_degree_core_periphery(DG2)
    
    rc = None
    while rc is None:
        try:
            rc = nx.rich_club_coefficient(DG2.to_undirected(), normalized=True, Q=100)
        except ZeroDivisionError:
            print "Oops!  Error! I am trying again!"
        
    enforc_betweenness_cp.append(count_core_periphery(k_bt))
    enforc_trophic_cp.append(count_core_periphery(k_tr))
    enforc_degree_cp.append(count_core_periphery(k_degr))
    enforc_rich_club.append(rc)
           
    print "Threshold: ", bn


# In[73]:


trc_cp = enforc_trophic_cp

plt.figure(figsize=(15,5))
plt.xlabel('Enforced basal nodes')
plt.ylabel('Trophic level core / periphery')
plt.plot([t for t in range(1, len(trc_cp) + 1)], [t[0] / float(t[1]) if t[1] != 0 else 0 for t in trc_cp])
plt.show()

plt.figure(figsize=(15,5))
plt.xlabel('Enforced basal nodes')
plt.ylabel('Graph size')
plt.plot([t for t in range(1, len(trc_cp)+1)], [t[0] + t[1] for t in trc_cp])
plt.show()


# # CORE PERIPHERY - merge

# In[120]:


font_size = 24

fig1, ax1_1 = plt.subplots(figsize=(10, 8))

ax1_1.plot([t * 0.1 for t in range(1, len(flow_degree_cp) + 1)], [t[0] / float(t[0]+t[1]) if t[1] != 0 else 0 for t in flow_degree_cp], 'b', label = "flow filter")
ax1_1.set_xlabel('Passenger flow filter', color='b', fontsize=font_size)
ax1_1.tick_params('x', colors='b', labelsize = font_size)


ax1_2 = ax1_1.twiny()
ax1_2.plot([t for t in range(1, len(enforc_degree_cp) + 1)], [t[0] / float(t[0]+t[1]) if t[1] != 0 else 0 for t in enforc_degree_cp], 'r', label = "basal nodes enforcement")
ax1_2.set_xlabel('Enforced basal nodes', color='r', fontsize=font_size)
ax1_2.tick_params('x', colors='r', labelsize = font_size)

ax1_1.tick_params('y', labelsize = font_size)

ax1_1.set_yticks([x/float(10) for x in range(11)])
ax1_2.set_yticks([x/float(10) for x in range(11)])

ax1_1.set_yticklabels([str(x*10)+'%' for x in range(11)])
ax1_1.set_ylabel('Degree core', fontsize=font_size)

#fig1.legend(fontsize=16, bbox_to_anchor=(0.96, 0.86))
fig1.tight_layout()


fig, ax1 = plt.subplots(figsize=(10, 8))

ax1.plot([t * 0.1 for t in range(0, len(flow_trophic_cp))], [t[0] / float(t[0]+t[1]) if t[0]+t[1] != 0 else 0 for t in flow_trophic_cp], 'b', label = "flow filter")
ax1.set_xlabel('Passenger flow filter', color='b', fontsize=font_size)
ax1.tick_params('x', colors='b', labelsize = font_size)

ax2 = ax1.twiny()
ax2.plot([t for t in range(1, len(enforc_trophic_cp) + 1)], [t[0] / float(t[0]+t[1]) if t[0]+t[1] != 0 else 0 for t in enforc_trophic_cp], 'r', label = "basal nodes enforcement")
ax2.set_xlabel('Enforced basal nodes', color='r', fontsize=font_size)
ax2.tick_params('x', colors='r', labelsize = font_size)

ax1.tick_params('y', labelsize = font_size)

ax1.set_yticks([x/float(10) for x in range(11)])
ax2.set_yticks([x/float(10) for x in range(11)])

ax1.set_yticklabels([str(x*10)+'%' for x in range(11)])
ax1.set_ylabel('Trophic core', fontsize=font_size)

#fig.legend(fontsize=16, bbox_to_anchor=(0.96, 0.86))
fig.tight_layout()


# # COMPANIES COMPARISON

# ## Trophic level

# In[121]:



q = {}
q_b = {}
size = {}

for t in range(3,9):
    threshold = t / 2.0
    for c in companies:

        #DG_filt = set_basal_nodes(DG_companies[c], number_basals = bn)
        #S, DG2 = compute_trophic_levels(DG_filt)

        DG_net = compute_net_network(DG_companies[c], threshold)
        DG2 = remove_disconnected_nodes(DG_net)
        if DG2.number_of_edges() == 0 or DG2.number_of_nodes() == 0:
            print("No net: ", c)
            continue

        S, DG2 = compute_trophic_levels(DG2)

        basal_nodes, top_nodes = get_basal_and_top_nodes(DG2)
        t_diff = trophic_diff(DG2)
        q.setdefault(c, []).append(incoherence_parameter(DG2, t_diff))
        q_b.setdefault(c, []).append(basal_ensemble_expectation(DG2, None))
        size[c] =  DG2.number_of_nodes()


# In[122]:


qb_ratio = {}
for key, value in q.iteritems():
    qb = [q[key][x] / float(q_b[key][x]) if q_b[key][x] != 0 else 0 for x in range(len(q[key]))]
    q[key] = [np.mean(q[key]), np.std(q[key])]
    q_b[key] = [np.mean(q_b[key]), np.std(q_b[key])]
    qb_ratio[key] = [np.mean(qb), np.std(qb)]
    


# In[123]:


del_array_q = []

min_nodes = 15

for key, value in delays.iteritems():
    if value[0] and key in size and size[key] >= min_nodes:
        temp = [key,value, q[key], size[key], q_b[key], qb_ratio[key]]
        del_array_q.append(temp)


del_array_q = sorted(del_array_q, key=lambda x: x[2][0]/float(x[4][0]), reverse=False)

fig, ax1 = plt.subplots(figsize=(10, 5))

ax1.set_xticks([x for x in range(len(del_array_q))], minor=False)
ax1.set_xticklabels([d[0].replace('"','') for d in del_array_q], fontdict=None, minor=False, rotation='vertical')


ax1.plot([d[5][0] for d in del_array_q], 'b')
ax1.set_ylabel('q / q^ ratio', color='b')
ax1.tick_params('y', colors='b')

ax2 = ax1.twinx()
ax2.plot([d[2][0] for d in del_array_q], 'g')
ax2.set_ylabel('q', color='g')
ax2.tick_params('y', colors='g')

# ax2 = ax1.twinx()
# ax2.plot([d[1][1] for d in del_array_q], 'r')
# ax2.set_ylabel('CaSL', color='r')
# ax2.tick_params('y', colors='r')

#ax3 = ax1.twinx()
# ax3.plot([d[1][2] for d in del_array_q], 'y')
# ax3.set_ylabel('Avg lateness', color='y')
# ax3.tick_params('y', colors='y')

ax3 = ax1.twinx()
ax3.plot([d[1][0] for d in del_array_q], 'r')
ax3.set_ylabel('PPM', color='r')
ax3.tick_params('y', colors='r')

fig.tight_layout()

plt.show()

plt.figure(figsize=(10, 5))
plt.bar([x for x in range(len(del_array_q))], [d[3] for d in del_array_q])
plt.xticks([x for x in range(len(del_array_q))], [d[0].replace('"','') for d in del_array_q], rotation='vertical', size=14)
plt.show()


# In[124]:


del_array = None
del_array_q = []

min_nodes = 15

for key, value in delays.iteritems():
    if value[0] and key in size and size[key] >= min_nodes:
        temp = [key,value, q[key][0], size[key], q[key][1], qb_ratio[key][0], qb_ratio[key][1]] # name, delays, q mean, size, q error, qb, qb_error
        del_array_q.append(temp)

# order per ppm
del_array_q = sorted(del_array_q, key=lambda x: x[1][0], reverse=False)


fig = plt.figure(figsize=(15,5))
ax = fig.add_subplot(111)
plt.xlabel('PPM', fontsize=16)
plt.ylabel('Incoherence parameter', fontsize=16)
A = [d[5] for d in del_array_q]
A_err = [d[6] for d in del_array_q]
B = [d[1][0] for d in del_array_q]
plt.errorbar(B, A, A_err,  fmt='o-')
plt.xticks(fontsize=16)
plt.yticks(fontsize=16)

txt = 0
for xy in zip(B, A):                                
    ax.annotate(del_array_q[txt][0].replace('"',''), xy=xy, textcoords='data', fontsize=16)
    txt += 1
plt.show()


# order per CaSL
del_array_q = sorted(del_array_q, key=lambda x: x[1][2], reverse=False)

fig = plt.figure(figsize=(15,5))
ax = fig.add_subplot(111)
plt.xlabel('CaSL', fontsize=16)
plt.ylabel('Incoherence parameter', fontsize=16)
A = [d[5] for d in del_array_q]
A_err = [d[6] for d in del_array_q]
B = [float(d[1][2]) for d in del_array_q]
plt.errorbar(B, A, A_err, fmt='o-')
plt.xticks(fontsize=16)
plt.yticks(fontsize=16)

txt = 0
for xy in zip(B, A):                          
    ax.annotate(del_array_q[txt][0].replace('"',''), xy=xy, textcoords='data', fontsize=14)
    txt += 1
plt.show()


# ## Merge

# In[136]:


font_size = 24

del_array_q = []
marker_arr = ['o','^','s','p','h']

min_nodes = 15

txt = 0
for key, value in delays.iteritems():
    if value[0] and key in size and size[key] >= min_nodes:
        temp = [key,value, q[key][0], size[key], q[key][1], qb_ratio[key][0], qb_ratio[key][1], marker_arr[txt]] # name, delays, q mean, size, q error, qb, qb_error, marker
        del_array_q.append(temp)
        txt += 1

fig, ax1 = plt.subplots(figsize=(15, 8))

# order per ppm
del_array_q = sorted(del_array_q, key=lambda x: x[1][0], reverse=False)

A = [d[5] for d in del_array_q]
A_err = [d[6] for d in del_array_q]
B = [d[1][0] for d in del_array_q]
M = [d[7] for d in del_array_q]
ax1.errorbar(B, A, A_err, fmt='o-')
ax1.set_xlabel('oPPM', color='b', fontsize=font_size)
ax1.tick_params('x', colors='b', labelsize = font_size)

txt = 0
for xy in zip(B, A):
    ax1.scatter(xy[0], xy[1], marker=M[txt], s=300, zorder=10, label=del_array_q[txt][0].replace('"',''), color='black')
    txt += 1
# order per CaSL
del_array_q = sorted(del_array_q, key=lambda x: x[1][2], reverse=False)

ax2 = ax1.twiny()
A = [d[5] for d in del_array_q]
A_err = [d[6] for d in del_array_q]
B = [float(d[1][2]) for d in del_array_q]
M = [d[7] for d in del_array_q]

ax2.errorbar(B, A, A_err, fmt='o-r')
ax2.set_xlabel('CaSL', color='r', fontsize=font_size)
ax2.tick_params('x', colors='r', labelsize = font_size)

txt = 0
for xy in zip(B, A):
    ax2.scatter(xy[0], xy[1], marker=M[txt], s=300, zorder=10, color='black')
    txt += 1

ax1.tick_params('y', labelsize = font_size)
ax1.set_ylabel('q/q^ ratio', fontsize=font_size)
ax1.legend(fontsize=font_size, loc=2)

fig.tight_layout()


# In[126]:


q_correlation = []

# q
# PPM correlation
q_correlation.append(pearsonr([d[2] for d in del_array_q], [d[1][0] for d in del_array_q])[0])
#CaSL correlation
q_correlation.append(pearsonr([d[2] for d in del_array_q], [d[1][2] for d in del_array_q])[0])

# q/q^
# PPM correlation
q_correlation.append(pearsonr([d[5] for d in del_array_q], [d[1][0] for d in del_array_q])[0])
#CaSL correlation
q_correlation.append(pearsonr([d[5] for d in del_array_q], [d[1][2] for d in del_array_q])[0])

# q q/q^ correlation
q_correlation.append(pearsonr([d[2] for d in del_array_q], [d[5] for d in del_array_q])[0])


print "---TROPHIC LEVEL---"
print "PPM"
print q_correlation[0]
print "CaSL"
print q_correlation[1]

print q_correlation


# # COMPANIES COMPARISON

# ##  core periphery

# In[155]:


betweenness_cp = {}
trophic_cp = {}
degree_cp = {}
rich_club_cp = {}

for t in range(3,9):
    threshold = t / 2.0
    for c in companies:
        DG_net = compute_net_network(DG_companies[c], threshold)
        DG2 = remove_disconnected_nodes(DG_net)
        if DG2.number_of_edges() == 0 or DG2.number_of_nodes() == 0:
            print("No net: ", c)
            continue

        S, DG2 = compute_trophic_levels(DG2)

        basal_nodes, top_nodes = get_basal_and_top_nodes(DG2)
        k_bt = get_betweenness_core_periphery(DG2)
        k_tr = get_trophic_core_periphery(DG2)
        k_degr = get_degree_core_periphery(DG2)
    
        rc = None
        if nx.number_of_nodes(DG2.to_undirected()) > 4:
            while rc is None:
                try:
                    rc = nx.rich_club_coefficient(DG2.to_undirected(), normalized=True)
                except ZeroDivisionError:
                    print "Oops!  Error! I am trying again!"
                except nx.NetworkXAlgorithmError:
                    print c
                    rc = []

        betweenness_cp.setdefault(c, []).append(count_core_periphery(k_bt))
        trophic_cp.setdefault(c, []).append(count_core_periphery(k_tr))
        degree_cp.setdefault(c, []).append(count_core_periphery(k_degr))
        rich_club_cp.setdefault(c, []).append(rc)


# In[156]:


trophic_cp_ratio = {}
for key, value in trophic_cp.iteritems(): 
    ratio = [x[0]/float(x[0]+x[1]) if x[0]+x[1] > 0 else np.nan for x in trophic_cp[key]]
    trophic_cp_ratio[key] = [np.nanmean(ratio), np.nanstd(ratio)]              

degree_cp_ratio = {}
for key, value in degree_cp.iteritems(): 
    ratio = [x[0]/float(x[0]+x[1]) if x[0]+x[1] > 0 else np.nan for x in degree_cp[key]]
    degree_cp_ratio[key] = [np.nanmean(ratio), np.nanstd(ratio)] 

rich_club_cp_mean = {}
for key, value in rich_club_cp.iteritems(): 
    rc_arr = {}
    rich_club_cp_mean[key] = {}
    max_k = max([max(x.keys()) if x else 0 for x in rich_club_cp[key]])
    for k in range(max_k + 1):
        k_arr = []
        if len(rich_club_cp[key]):
            k_arr = [x[k] if x and k in x else np.nan for x in rich_club_cp[key]]
    
        if len(k_arr) and k_arr.count(np.nan) != len(k_arr):
            rich_club_cp_mean[key][k] = [np.nanmean(k_arr), np.nanstd(k_arr)] 
        else:
            rich_club_cp_mean[key] = [None, None] 


# ## Merge

# In[225]:


def plot_cp_comparison(cp_ratio, y_axis_name, ylim=(-0.1,1)):
    font_size = 24

    del_array_cp = []
    marker_arr = ['o','^','s','p','h']

    min_nodes = 15

    txt = 0
    for key, value in delays.iteritems():
        if value[0] and key in size and size[key] >= min_nodes:
            temp = [key,value, cp_ratio[key], size[key], marker_arr[txt]]
            del_array_cp.append(temp)
            txt += 1

    fig, ax1 = plt.subplots(figsize=(15, 8))

    #order per PPM ratio
    del_array_cp = sorted(del_array_cp, key=lambda x: x[1][0], reverse=False)

    A = [d[2][0] for d in del_array_cp]
    A_err = [d[2][1] for d in del_array_cp]
    B = [d[1][0] for d in del_array_cp]
    M = [d[4] for d in del_array_cp]
    ax1.errorbar(B, A, A_err, fmt='-')
    ax1.set_xlabel('oPPM', color='b', fontsize=font_size)
    ax1.tick_params('x', colors='b', labelsize = font_size)

    txt = 0
    for xy in zip(B, A):
        ax1.scatter(xy[0], xy[1], marker=M[txt], s=300, zorder=10, label=del_array_cp[txt][0].replace('"',''), color='black')
        txt += 1


    #order per CaSL ratio
    del_array_cp = sorted(del_array_cp, key=lambda x: x[1][2], reverse=False)

    ax2 = ax1.twiny()
    A = [d[2][0] for d in del_array_cp]
    A_err = [d[2][1] for d in del_array_cp]
    B = [float(d[1][2]) for d in del_array_cp]
    M = [d[4] for d in del_array_cp]
    ax2.errorbar(B, A, A_err, fmt='-r')
    ax2.set_xlabel('CaSL', color='r', fontsize=font_size)
    ax2.tick_params('x', colors='r', labelsize = font_size)

    txt = 0
    for xy in zip(B, A):
        ax2.scatter(xy[0], xy[1], marker=M[txt], s=300, zorder=10, color = 'black')
        txt += 1


    ax1.tick_params('y', labelsize = font_size)
    ax1.set_ylabel(y_axis_name, fontsize=font_size)
    ax1.legend(fontsize=font_size)

    if ylim[1] > 10:
        ax1.set_yticks([x for x in range(0,ylim[1] + 1, 5)])
        ax2.set_yticks([x for x in range(0,ylim[1] + 1, 5)])
        ax1.set_yticklabels([str(x)+'%' for x in range(0, ylim[1] + 1, 5)])
    else:
        ax1.set_yticks([x/float(10) for x in range(0,ylim[1] * 10 + 1)])
        ax2.set_yticks([x/float(10) for x in range(0,ylim[1] * 10 + 1)])
        ax1.set_yticklabels([str(x*10)+'%' for x in range(0, ylim[1] * 10 + 1)])
    plt.ylim(ylim)

    fig.tight_layout()
    
    return del_array_cp



# In[226]:


del_array_trophic_cp = plot_cp_comparison(trophic_cp_ratio, 'Trophic core')


# In[227]:


del_array_degree_cp = plot_cp_comparison(degree_cp_ratio, 'Degree core')


# In[161]:


size = {'"London Overground"': 109, '"Chiltern Railways"': 18, '"Great Western Railway"': 18, '"Southeastern"': 63, '"South West Trains"': 91}

rc_array_cp = []

min_nodes = 15

plt.figure(figsize=(10, 10))
for key, value in rich_club_cp_mean.iteritems():
    if value[0] and key in size and size[key] >= min_nodes:
        print key
        max_k = max(value.keys())
        rc_array_cp.append([value[x][0] for x in range(max_k+1)])
        
        plt.plot(rc_array_cp[-1], label=key)

plt.legend()


# In[162]:


font_size = 24
size = {'"London Overground"': 109, '"Chiltern Railways"': 18, '"Great Western Railway"': 18, '"Southeastern"': 63, '"South West Trains"': 91}
marker_arr = ['o','^','s','p','h']
rc_array_cp = []
min_nodes = 15
plt.figure(figsize=(10, 10))
txt = 0
for key, value in rich_club_cp_mean.iteritems():
    if value[0] and key in size and size[key] >= min_nodes:
        max_k = max(value.values(), key=lambda x: x[0])
        rc_array_cp.append([key, max_k, delays[key], marker_arr[txt]])
        txt += 1

fig, ax1 = plt.subplots(figsize=(15, 8))

#order per PPM ratio
rc_array_cp = sorted(rc_array_cp, key=lambda x: x[2][0], reverse=False)

A = [d[1][0] for d in rc_array_cp]
A_err = [d[1][1] for d in rc_array_cp]
B = [d[2][0] for d in rc_array_cp]
M = [d[3] for d in rc_array_cp]

ax1.errorbar(B, A, A_err, fmt='-')
ax1.set_xlabel('oPPM', color='b', fontsize=font_size)
ax1.tick_params('x', colors='b', labelsize = font_size)

txt = 0
for xy in zip(B, A):
    ax1.scatter(xy[0], xy[1], marker=M[txt], s=300, zorder=10, label=rc_array_cp[txt][0].replace('"',''), color='black')
    txt += 1

#order per CaSL ratio
rc_array_cp = sorted(rc_array_cp, key=lambda x: float(x[2][2]), reverse=False)

ax2 = ax1.twiny()

A = [d[1][0] for d in rc_array_cp]
A_err = [d[1][1] for d in rc_array_cp]
B = [float(d[2][2]) for d in rc_array_cp]
M = [d[3] for d in rc_array_cp]

ax2.errorbar(B, A, A_err, fmt='-r')
ax2.set_xlabel('CaSL', color='r', fontsize=font_size)
ax2.tick_params('x', colors='r', labelsize = font_size)

txt = 0
for xy in zip(B, A):
    ax2.scatter(xy[0], xy[1], marker=M[txt], s=300, zorder=10, label=rc_array_cp[txt][0].replace('"',''), color='black')
    txt += 1

ax1.tick_params('y', labelsize = font_size)
ax1.set_ylabel('Rich club coefficient', fontsize=font_size)
ax1.legend(fontsize=font_size, loc=2)

fig.tight_layout()


# In[233]:


#DEL_ARRAY = name, delays_array[PPM, CaSL, ...], cp-trophic, cp-betweeness, network size
cp_correlation = []
rc_correlation = []

# PPM - trophic level correlation
cp_correlation.append(pearsonr([d[2][0] for d in del_array_trophic_cp], [d[1][0] for d in del_array_trophic_cp])[0])
#CaSL - trophic level correlation
cp_correlation.append(pearsonr([d[2][0] for d in del_array_trophic_cp], [d[1][2] for d in del_array_trophic_cp])[0])
# PPM - betweenees correlation
#cp_correlation.append(pearsonr([d[3] for d in del_array_cp], [d[1][0] for d in del_array_cp])[0])
#CaSL - betweenees correlation
#cp_correlation.append(pearsonr([d[3] for d in del_array_cp], [d[1][2] for d in del_array_cp])[0])

# PPM - degree level correlation
cp_correlation.append(pearsonr([d[2][0] for d in del_array_degree_cp], [d[1][0] for d in del_array_degree_cp])[0])
#CaSL - degree level correlation
cp_correlation.append(pearsonr([d[2][0] for d in del_array_degree_cp], [d[1][2] for d in del_array_degree_cp])[0])


# PPM - degree level correlation
rc_correlation.append(pearsonr([d[1][0] for d in rc_array_cp], [d[2][0] for d in rc_array_cp])[0])
#CaSL - degree level correlation
rc_correlation.append(pearsonr([d[1][0] for d in rc_array_cp], [d[2][2] for d in rc_array_cp])[0])


print "---TROPHIC CORRELATION---"
print "PPM"
print cp_correlation[0]
print "CaSL"
print cp_correlation[1]

print "---DEGREE CORRELATION---"
print "PPM"
print cp_correlation[2]
print "CaSL"
print cp_correlation[3]

print "---RICH-CLUB CORRELATION---"
print "PPM"
print rc_correlation[0]
print "CaSL"
print rc_correlation[1]

#print "---BETWEENESS---"
#print "PPM"
#print cp_correlation[2]
#print "CaSL"
#print cp_correlation[3]

print cp_correlation
print rc_correlation


# # Q - CP and PPM - CaSL correlation

# In[269]:


q_trophic_cp_correlation = []
# q - cp
q_trophic_cp_correlation.append(pearsonr([d[2] for d in sorted(del_array_q, key=lambda x: x[0])], [d[2][0] for d in sorted(del_array_trophic_cp, key=lambda x: x[0])])[0])
# q/q^ - cp
q_trophic_cp_correlation.append(pearsonr([d[5] for d in sorted(del_array_q, key=lambda x: x[0])], [d[2][0] for d in sorted(del_array_trophic_cp, key=lambda x: x[0])])[0])


q_degree_cp_correlation = []
# q - cp
q_degree_cp_correlation.append(pearsonr([d[2] for d in sorted(del_array_q, key=lambda x: x[0])], [d[2][0] for d in sorted(del_array_degree_cp, key=lambda x: x[0])])[0])
# q/q^ - cp
q_degree_cp_correlation.append(pearsonr([d[5] for d in sorted(del_array_q, key=lambda x: x[0])], [d[2][0] for d in sorted(del_array_degree_cp, key=lambda x: x[0])])[0])

q_rc_correlation = []
# q - cp
q_rc_correlation.append(pearsonr([d[2] for d in sorted(del_array_q, key=lambda x: x[0])], [d[1][0] for d in sorted(rc_array_cp, key=lambda x: x[0])])[0])
# q/q^ - cp
q_rc_correlation.append(pearsonr([d[5] for d in sorted(del_array_q, key=lambda x: x[0])], [d[1][0] for d in sorted(rc_array_cp, key=lambda x: x[0])])[0])


ppm_casl_correlation = []
ppm_casl_correlation.append(pearsonr([d[1][0] for d in sorted(del_array_q, key=lambda x: x[0])], [d[1][2] for d in sorted(del_array_q, key=lambda x: x[0])])[0])
#ppm_casl_correlation.append(pearsonr([d[1][0] for d in del_array_cp], [d[1][1] for d in del_array_cp])[0])

degree_trophic_cp_correlation = []
degree_trophic_cp_correlation.append(pearsonr([d[2][0] for d in sorted(del_array_degree_cp, key=lambda x: x[0])], [d[2][0] for d in sorted(del_array_trophic_cp, key=lambda x: x[0])])[0])




print(q_trophic_cp_correlation)
print(q_degree_cp_correlation)
print(q_rc_correlation)
print(ppm_casl_correlation)
print(degree_trophic_cp_correlation)


# In[ ]:


plt.plot([d[2] for d in sorted(del_array_q, key=lambda x: x[0])])
plt.plot([d[2] for d in sorted(del_array_cp, key=lambda x: x[0])])


# # Data for Gephi

# In[ ]:


threshold = 1

DG_study = DG_companies['"Great Western Railway"']
DG_net = compute_net_network(DG_study, threshold)
DG_disc = remove_disconnected_nodes(DG_net)

S, DG2 = compute_trophic_levels(DG_disc)

k_tr = get_trophic_core_periphery(DG2)
core, periphery = count_core_periphery(k_tr)
#  k_bt = get_betweenness_core_periphery(DG2)
# betweenness_cp[c] = count_core_periphery(k_bt)
  
border =  max(k_tr, key=lambda x: x[1])

is_core = True
for node in k_tr:
    if is_core:
        DG2.nodes()[node[0][0]]['core'] = 1
    else:
        DG2.nodes()[node[0][0]]['core'] = 0
    if node == border:
        is_core = False


for node in DG2.nodes(data=True):
    if node[0] in gps_coordinates:
        DG2.nodes()[node[0]]['lat'] = float(gps_coordinates[node[0]][1])
        DG2.nodes()[node[0]]['lng'] = float(gps_coordinates[node[0]][2])

nx.write_gexf(DG2, "Great Western Railway.gexf", version="1.2draft")


# # Multiplex nodes

# In[ ]:


multiplex_nodes


# # Robustness: node removal

# In[165]:


largest_comp_arr = {}
largest_comp = {}

for k,g in DG_companies.iteritems():
    largest_comp_arr[k] = []
    
    for _ in range(500):
        company_g = g.copy().to_undirected()
        lc = []
        while len(company_g.nodes())>1:
           # print len(complany_g.nodes()), "-",len(max(nx.connected_components(complany_g), key=len))
            lc.append(len(max(nx.connected_components(company_g), key=len)))
            node = np.random.choice(company_g.nodes())
            company_g.remove_node(node)
        largest_comp_arr[k].append(lc)
        
for k,v in largest_comp_arr.iteritems():
    perc = [[x/float(vv[0]) for x in vv] for vv in v]
    largest_comp[k] = [np.nanmean(perc, axis=0), np.nanstd(perc, axis=0)]
       


# In[276]:


font_size = 24

size = {'"London Overground"': 109, '"Chiltern Railways"': 18, '"Great Western Railway"': 18, '"Southeastern"': 63, '"South West Trains"': 91}

plt.figure(figsize=(15, 7))

min_nodes = 15

for key, value in largest_comp.iteritems():
    if key in size and size[key] >= min_nodes:
        plt.errorbar([s/float(len(value[0])-1) for s in range(len(value[0]))],
                     [dim for dim in value[0]],
                     [dim for dim in value[1]],
                     linewidth=3, elinewidth=1,label=key.replace('"',''))
  
    
plt.ylabel('Connected network', fontsize=font_size)
plt.xlabel('Nodes removed', fontsize=font_size)
plt.xticks(fontsize=16)
lbl = [str(x*10)+'%' for x in range(11)]
plt.yticks([x/float(10) for x in range(len(lbl))] ,lbl, fontsize=font_size)
plt.xticks([x/float(10) for x in range(len(lbl))] ,lbl, fontsize=font_size)

# red line
plt.plot([s/float(len(value)-1) for s in range(len(value))], [0.5 for _ in range(len(value))], linestyle="--", c='r')


plt.legend(fontsize=font_size)


# In[234]:


connectivity = {}

for key, value in largest_comp_arr.iteritems():
    perc_over_50_arr = []
    for v in value:
        perc_over_50 = len(filter(lambda x: x/float(v[0]) > 0.5, v))
        perc_over_50_arr.append(perc_over_50)

    connectivity[key] = [np.nanmean(perc_over_50_arr), np.nanstd(perc_over_50_arr), len(value[0])]


# In[235]:


connectivity_arr = plot_cp_comparison(connectivity, 'Nodes removed', ylim=(-0.1,30))


# In[236]:


connectivity_correlation = []

# PPM - connectivity level correlation
connectivity_correlation.append(pearsonr([d[2][0] for d in connectivity_arr], [d[1][0] for d in connectivity_arr])[0])
#CaSL - connectivity level correlation
connectivity_correlation.append(pearsonr([d[2][0] for d in connectivity_arr], [d[1][2] for d in connectivity_arr])[0])

print "---CONNECTIVITY CORRELATION---"
print "PPM"
print connectivity_correlation[0]
print "CaSL"
print connectivity_correlation[1]

print connectivity_correlation


# In[275]:


q_connectivity_correlation = []
# q - cp
q_connectivity_correlation.append(pearsonr([d[2] for d in sorted(del_array_q, key=lambda x: x[0])], [d[1][0] for d in sorted(connectivity_arr, key=lambda x: x[0])])[0])
# q/q^ - cp
q_connectivity_correlation.append(pearsonr([d[5] for d in sorted(del_array_q, key=lambda x: x[0])], [d[1][0] for d in sorted(connectivity_arr, key=lambda x: x[0])])[0])

rc_connectivity_correlation = []
rc_connectivity_correlation.append(pearsonr([d[1][0] for d in sorted(rc_array_cp, key=lambda x: x[0])], [d[1][0] for d in sorted(connectivity_arr, key=lambda x: x[0])])[0])

# rich-club
degree_cp_rc_correlation = []
degree_cp_rc_correlation.append(pearsonr([d[1][0] for d in sorted(rc_array_cp, key=lambda x: x[0])], [d[2][0] for d in sorted(del_array_degree_cp, key=lambda x: x[0])])[0])
trophic_cp_rc_correlation = []
trophic_cp_rc_correlation.append(pearsonr([d[1][0] for d in sorted(rc_array_cp, key=lambda x: x[0])], [d[2][0] for d in sorted(del_array_trophic_cp, key=lambda x: x[0])])[0])

# connectivity
degree_cp_connectivity_correlation = []
degree_cp_connectivity_correlation.append(pearsonr([d[2][0] for d in sorted(connectivity_arr, key=lambda x: x[0])], [d[2][0] for d in sorted(del_array_degree_cp, key=lambda x: x[0])])[0])
trophic_cp_connectivity_correlation = []
trophic_cp_connectivity_correlation.append(pearsonr([d[2][0] for d in sorted(connectivity_arr, key=lambda x: x[0])], [d[2][0] for d in sorted(del_array_trophic_cp, key=lambda x: x[0])])[0])



print(q_connectivity_correlation)
print(rc_connectivity_correlation)
print(degree_cp_rc_correlation)
print(trophic_cp_rc_correlation)
print(degree_cp_connectivity_correlation)
print(trophic_cp_connectivity_correlation)


# # Correlation analysis

# In[237]:


# Multiplex nodes
min_nodes = 15
multiplex_nodes = {}

for key, value in DG_companies.iteritems():
    mp = 0
    for key_iter, value_iter in DG_companies.iteritems():
        if key != key_iter:
            mp += len(list(set(value.nodes()).intersection(value_iter.nodes())))
    multiplex_nodes[key] = mp


# In[256]:


del_array_info = []

min_nodes = 15

for key, value in delays.iteritems():
    if value[0] and key in size and size[key] >= min_nodes:
        max_k = max(rich_club_cp_mean[key].values(), key=lambda x: x[0])
        temp = [key, q[key][0], trophic_cp_ratio[key][0], info[key], delays[key], multiplex_nodes[key], qb_ratio[key][0], max_k[0], connectivity[key][0], degree_cp_ratio[key][0]]
        del_array_info.append(temp)


del_array_info = sorted(del_array_info, key=lambda x: x[0], reverse=False)


# In[259]:


qratio_info_correlation = []
qratio_info_correlation.append(pearsonr([d[6] for d in del_array_info], [d[3][0] for d in del_array_info])[0])
print("Q^-employees", qratio_info_correlation[-1])
qratio_info_correlation.append(pearsonr([d[6] for d in del_array_info], [d[3][1] for d in del_array_info])[0])
print("Q^-stations", qratio_info_correlation[-1])
qratio_info_correlation.append(pearsonr([d[6] for d in del_array_info], [d[3][2] for d in del_array_info])[0])
print("Q^-trains", qratio_info_correlation[-1])
qratio_info_correlation.append(pearsonr([d[6] for d in del_array_info], [d[3][3] for d in del_array_info])[0])
print("Q^-passengers", qratio_info_correlation[-1])
qratio_info_correlation.append(pearsonr([d[6] for d in del_array_info], [d[5] for d in del_array_info])[0])
print("Q^-multiplex", qratio_info_correlation[-1])


rc_info_correlation = []
rc_info_correlation.append(pearsonr([d[7] for d in del_array_info], [d[3][0] for d in del_array_info])[0])
rc_info_correlation.append(pearsonr([d[7] for d in del_array_info], [d[3][1] for d in del_array_info])[0])
rc_info_correlation.append(pearsonr([d[7] for d in del_array_info], [d[3][2] for d in del_array_info])[0])
rc_info_correlation.append(pearsonr([d[7] for d in del_array_info], [d[3][3] for d in del_array_info])[0])
rc_info_correlation.append(pearsonr([d[7] for d in del_array_info], [d[5] for d in del_array_info])[0])


connect_info_correlation = []
connect_info_correlation.append(pearsonr([d[8] for d in del_array_info], [d[3][0] for d in del_array_info])[0])
connect_info_correlation.append(pearsonr([d[8] for d in del_array_info], [d[3][1] for d in del_array_info])[0])
connect_info_correlation.append(pearsonr([d[8] for d in del_array_info], [d[3][2] for d in del_array_info])[0])
connect_info_correlation.append(pearsonr([d[8] for d in del_array_info], [d[3][3] for d in del_array_info])[0])
connect_info_correlation.append(pearsonr([d[8] for d in del_array_info], [d[5] for d in del_array_info])[0])

degree_cp_info_correlation = []
degree_cp_info_correlation.append(pearsonr([d[9] for d in del_array_info], [d[3][0] for d in del_array_info])[0])
degree_cp_info_correlation.append(pearsonr([d[9] for d in del_array_info], [d[3][1] for d in del_array_info])[0])
degree_cp_info_correlation.append(pearsonr([d[9] for d in del_array_info], [d[3][2] for d in del_array_info])[0])
degree_cp_info_correlation.append(pearsonr([d[9] for d in del_array_info], [d[3][3] for d in del_array_info])[0])
degree_cp_info_correlation.append(pearsonr([d[9] for d in del_array_info], [d[5] for d in del_array_info])[0])


q_info_correlation = []
q_info_correlation.append(pearsonr([d[1] for d in del_array_info], [d[3][0] for d in del_array_info])[0])
print("Q-employees", q_info_correlation[-1])
q_info_correlation.append(pearsonr([d[1] for d in del_array_info], [d[3][1] for d in del_array_info])[0])
print("Q-stations", q_info_correlation[-1])
q_info_correlation.append(pearsonr([d[1] for d in del_array_info], [d[3][2] for d in del_array_info])[0])
print("Q-trains", q_info_correlation[-1])
q_info_correlation.append(pearsonr([d[1] for d in del_array_info], [d[3][3] for d in del_array_info])[0])
print("Q-passengers", q_info_correlation[-1])
q_info_correlation.append(pearsonr([d[1] for d in del_array_info], [d[5] for d in del_array_info])[0])
print("Q-multiplex", q_info_correlation[-1])

trophic_cp_info_correlation = []
trophic_cp_info_correlation.append(pearsonr([d[2] for d in del_array_info], [d[3][0] for d in del_array_info])[0])
print("CP-employees", trophic_cp_info_correlation[-1])
trophic_cp_info_correlation.append(pearsonr([d[2] for d in del_array_info], [d[3][1] for d in del_array_info])[0])
print("CP-stations", trophic_cp_info_correlation[-1])
trophic_cp_info_correlation.append(pearsonr([d[2] for d in del_array_info], [d[3][2] for d in del_array_info])[0])
print("CP-trains", trophic_cp_info_correlation[-1])
trophic_cp_info_correlation.append(pearsonr([d[2] for d in del_array_info], [d[3][3] for d in del_array_info])[0])
print("CP-passengers", trophic_cp_info_correlation[-1])
trophic_cp_info_correlation.append(pearsonr([d[2] for d in del_array_info], [d[5] for d in del_array_info])[0])
print("CP-multiplex", trophic_cp_info_correlation[-1])

ppm_info_correlation = []
ppm_info_correlation.append(pearsonr([d[4][0] for d in del_array_info], [d[3][0] for d in del_array_info])[0])
print("PPM-employees", ppm_info_correlation[-1])
ppm_info_correlation.append(pearsonr([d[4][0] for d in del_array_info], [d[3][1] for d in del_array_info])[0])
print("PPM-stations", ppm_info_correlation[-1])
ppm_info_correlation.append(pearsonr([d[4][0] for d in del_array_info], [d[3][2] for d in del_array_info])[0])
print("PPM-trains", ppm_info_correlation[-1])
ppm_info_correlation.append(pearsonr([d[4][0] for d in del_array_info], [d[3][3] for d in del_array_info])[0])
print("PPM-passengers", ppm_info_correlation[-1])
ppm_info_correlation.append(pearsonr([d[4][0] for d in del_array_info], [d[5] for d in del_array_info])[0])
print("PPM-multiplex", ppm_info_correlation[-1])

casl_info_correlation = []
casl_info_correlation.append(pearsonr([d[4][2] for d in del_array_info], [d[3][0] for d in del_array_info])[0])
print("CaSL-employees", casl_info_correlation[-1])
casl_info_correlation.append(pearsonr([d[4][2] for d in del_array_info], [d[3][1] for d in del_array_info])[0])
print("CaSL-stations", casl_info_correlation[-1])
casl_info_correlation.append(pearsonr([d[4][2] for d in del_array_info], [d[3][2] for d in del_array_info])[0])
print("CaSL-trains", casl_info_correlation[-1])
casl_info_correlation.append(pearsonr([d[4][2] for d in del_array_info], [d[3][3] for d in del_array_info])[0])
print("CaSL-passengers", casl_info_correlation[-1])
casl_info_correlation.append(pearsonr([d[4][2] for d in del_array_info], [d[5] for d in del_array_info])[0])
print("CaSL-multiplex", casl_info_correlation[-1])


# In[ ]:


employees_info_correlation = []
employees_info_correlation.append(pearsonr([d[3][0] for d in del_array_info], [d[3][0] for d in del_array_info])[0])
print("employees-employees", employees_info_correlation[-1])
employees_info_correlation.append(pearsonr([d[3][0] for d in del_array_info], [d[3][1] for d in del_array_info])[0])
print("employees-stations", employees_info_correlation[-1])
employees_info_correlation.append(pearsonr([d[3][0] for d in del_array_info], [d[3][2] for d in del_array_info])[0])
print("employees-trains", employees_info_correlation[-1])
employees_info_correlation.append(pearsonr([d[3][0] for d in del_array_info], [d[3][3] for d in del_array_info])[0])
print("employees-passengers", employees_info_correlation[-1])
employees_info_correlation.append(pearsonr([d[3][0] for d in del_array_info], [d[5] for d in del_array_info])[0])
print("employees-multiplex", employees_info_correlation[-1])


stations_info_correlation = []
stations_info_correlation.append(pearsonr([d[3][1] for d in del_array_info], [d[3][0] for d in del_array_info])[0])
print("stations-employees", stations_info_correlation[-1])
stations_info_correlation.append(pearsonr([d[3][1] for d in del_array_info], [d[3][1] for d in del_array_info])[0])
print("stations-stations", stations_info_correlation[-1])
stations_info_correlation.append(pearsonr([d[3][1] for d in del_array_info], [d[3][2] for d in del_array_info])[0])
print("stations-trains", stations_info_correlation[-1])
stations_info_correlation.append(pearsonr([d[3][1] for d in del_array_info], [d[3][3] for d in del_array_info])[0])
print("stations-passengers", stations_info_correlation[-1])
stations_info_correlation.append(pearsonr([d[3][1] for d in del_array_info], [d[5] for d in del_array_info])[0])
print("stations-multiplex", stations_info_correlation[-1])

trains_info_correlation = []
trains_info_correlation.append(pearsonr([d[3][2] for d in del_array_info], [d[3][0] for d in del_array_info])[0])
print("trains-employees", trains_info_correlation[-1])
trains_info_correlation.append(pearsonr([d[3][2] for d in del_array_info], [d[3][1] for d in del_array_info])[0])
print("trains-stations", trains_info_correlation[-1])
trains_info_correlation.append(pearsonr([d[3][2] for d in del_array_info], [d[3][2] for d in del_array_info])[0])
print("trains-trains", trains_info_correlation[-1])
trains_info_correlation.append(pearsonr([d[3][2] for d in del_array_info], [d[3][3] for d in del_array_info])[0])
print("trains-passengers", trains_info_correlation[-1])
trains_info_correlation.append(pearsonr([d[3][2] for d in del_array_info], [d[5] for d in del_array_info])[0])
print("trains-multiplex", trains_info_correlation[-1])

passengers_info_correlation = []
passengers_info_correlation.append(pearsonr([d[3][3] for d in del_array_info], [d[3][0] for d in del_array_info])[0])
print("passengers-employees", passengers_info_correlation[-1])
passengers_info_correlation.append(pearsonr([d[3][3] for d in del_array_info], [d[3][1] for d in del_array_info])[0])
print("passengers-stations", passengers_info_correlation[-1])
passengers_info_correlation.append(pearsonr([d[3][3] for d in del_array_info], [d[3][2] for d in del_array_info])[0])
print("passengers-trains", passengers_info_correlation[-1])
passengers_info_correlation.append(pearsonr([d[3][3] for d in del_array_info], [d[3][3] for d in del_array_info])[0])
print("passengers-passengers", passengers_info_correlation[-1])
passengers_info_correlation.append(pearsonr([d[3][3] for d in del_array_info], [d[5] for d in del_array_info])[0])
print("passengers-multiplex", passengers_info_correlation[-1])

